package com.sun.xml.internal.ws.handler;

import com.sun.xml.internal.ws.api.pipe.helper.AbstractTubeImpl;
import com.sun.xml.internal.ws.api.message.Packet;
import com.sun.xml.internal.ws.api.handler.MessageHandler;
import java.util.List;
import com.sun.xml.internal.ws.client.HandlerConfiguration;
import java.util.HashSet;
import com.sun.xml.internal.ws.binding.BindingImpl;
import java.util.ArrayList;
import javax.xml.ws.handler.MessageContext;
import com.sun.xml.internal.ws.api.message.Attachment;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.message.AttachmentSet;
import com.sun.xml.internal.ws.message.DataHandlerAttachment;
import javax.activation.DataHandler;
import java.util.Map;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.api.pipe.helper.AbstractFilterTubeImpl;
import com.sun.xml.internal.ws.api.pipe.TubeCloner;
import com.sun.xml.internal.ws.api.pipe.Tube;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.xml.internal.ws.api.WSBinding;
import com.sun.istack.internal.Nullable;
import java.util.Set;
import com.sun.xml.internal.ws.api.model.SEIModel;

public class ClientMessageHandlerTube extends HandlerTube
{
    private SEIModel seiModel;
    private Set<String> roles;
    
    public ClientMessageHandlerTube(@Nullable SEIModel seiModel, WSBinding binding, WSDLPort port, Tube next) {
        super(next, port, binding);
        this.seiModel = seiModel;
    }
    
    private ClientMessageHandlerTube(ClientMessageHandlerTube that, TubeCloner cloner) {
        super(that, cloner);
        this.seiModel = that.seiModel;
    }
    
    public AbstractFilterTubeImpl copy(TubeCloner cloner) {
        return new ClientMessageHandlerTube(this, cloner);
    }
    
    @Override
    void callHandlersOnResponse(MessageUpdatableContext context, boolean handleFault) {
        try {
            this.processor.callHandlersResponse(HandlerProcessor.Direction.INBOUND, context, handleFault);
        }
        catch (WebServiceException wse) {
            throw wse;
        }
        catch (RuntimeException re) {
            throw new WebServiceException(re);
        }
    }
    
    @Override
    boolean callHandlersOnRequest(MessageUpdatableContext context, boolean isOneWay) {
        Map<String, DataHandler> atts = (Map)context.get("javax.xml.ws.binding.attachments.outbound");
        AttachmentSet attSet = context.packet.getMessage().getAttachments();
        for (Map.Entry<String, DataHandler> entry : atts.entrySet()) {
            String cid = (String)entry.getKey();
            if (attSet.get(cid) == null) {
                Attachment att = new DataHandlerAttachment(cid, (DataHandler)atts.get(cid));
                attSet.add(att);
            }
            else {
                continue;
            }
        }
        boolean handlerResult;
        try {
            handlerResult = this.processor.callHandlersRequest(HandlerProcessor.Direction.OUTBOUND, context, !isOneWay);
        }
        catch (WebServiceException wse) {
            this.remedyActionTaken = true;
            throw wse;
        }
        catch (RuntimeException re) {
            this.remedyActionTaken = true;
            throw new WebServiceException(re);
        }
        if (!handlerResult) {
            this.remedyActionTaken = true;
        }
        return handlerResult;
    }
    
    @Override
    void closeHandlers(MessageContext mc) {
        this.closeClientsideHandlers(mc);
    }
    
    @Override
    void setUpProcessor() {
        if (this.handlers == null) {
            this.handlers = new ArrayList();
            HandlerConfiguration handlerConfig = ((BindingImpl)this.getBinding()).getHandlerConfig();
            List<MessageHandler> msgHandlersSnapShot = handlerConfig.getMessageHandlers();
            if (!msgHandlersSnapShot.isEmpty()) {
                this.handlers.addAll(msgHandlersSnapShot);
                (this.roles = new HashSet()).addAll(handlerConfig.getRoles());
                this.processor = new SOAPHandlerProcessor(true, this, this.getBinding(), this.handlers);
            }
        }
    }
    
    @Override
    MessageUpdatableContext getContext(Packet p) {
        MessageHandlerContextImpl context = new MessageHandlerContextImpl(this.seiModel, this.getBinding(), this.port, p, this.roles);
        return context;
    }
}
